package engine;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dom.Order;
import dom.PizzaTypeSummary;
import dom2app.PizzaTypeSummaryInterface;

public class PizzariaEngine {
	private List<Order> allOrders;

	public PizzariaEngine() {
		this.allOrders = new ArrayList<Order>();
	}

	public boolean addOrder(String type, int qty){
		//TODO intentionally omitted a check on empty type string, to show how exception handling works
		//if((qty>0) && (!type.isEmpty()))
		if(qty>0) {
			Order o = new Order(type, qty);
			return this.allOrders.add(o);
		}
		return false;
	}

	public List<String> produceReport(){
		Map<String, PizzaTypeSummary> map = new HashMap<String, PizzaTypeSummary>();

		for(Order o: this.allOrders) {
			String type = o.getType();
			int qtyPizzas = o.getQuantity();
			int qtyOrders = 1;
			int orderPrice = o.getOrderPrice();

			PizzaTypeSummary pts = null;
			if (!map.containsKey(type)) {
				boolean correctCreation = true;
				try{
					pts = new PizzaTypeSummary(type, qtyOrders, qtyPizzas, orderPrice);
				}
				catch (IllegalArgumentException e) {
					System.err.println(e.getLocalizedMessage());
					correctCreation = false;
				}
				if(correctCreation) {
					map.put(type, pts);
				}
			}
			else {
				pts = map.get(type);
				pts.setNumberOfOrders( pts.getNumberOfOrders() + qtyOrders);
				pts.setNumberOfPizzas( pts.getNumberOfPizzas() + qtyPizzas);
				pts.setSummaryPrice(pts.getSummaryPrice() + orderPrice);
			}
		}//end for

		List<String> results = new ArrayList<String>();
		for(PizzaTypeSummary t: map.values()) {
			results.add(t.toString());
		}
		return results;
	}

	public Collection<PizzaTypeSummaryInterface> produceSummaryReport(){
		Map<String, PizzaTypeSummary> map = new HashMap<String, PizzaTypeSummary>();

		for(Order o: this.allOrders) {
			String type = o.getType();
			int qtyPizzas = o.getQuantity();
			int qtyOrders = 1;
			int orderPrice = o.getOrderPrice();

			PizzaTypeSummary pts = null;
			if (!map.containsKey(type)) {
				boolean correctCreation = true;
				try{
					pts = new PizzaTypeSummary(type, qtyOrders, qtyPizzas, orderPrice);
				}
				catch (IllegalArgumentException e) {
					System.err.println(e.getLocalizedMessage());
					correctCreation = false;
				}
				if(correctCreation) {
					map.put(type, pts);
				}
				
			}
			else {
				pts = map.get(type);
				pts.setNumberOfOrders( pts.getNumberOfOrders() + qtyOrders);
				pts.setNumberOfPizzas( pts.getNumberOfPizzas() + qtyPizzas);
				pts.setSummaryPrice(pts.getSummaryPrice() + orderPrice);
			}
		}//end for

		Collection<PizzaTypeSummaryInterface> result = new ArrayList<PizzaTypeSummaryInterface>();
		result.addAll(map.values());

		return result;
	}

	/**
	 * package private -- only classes from this package can see it
	 * 
	 * for example: test classes
	 * 
	 * @return a defensive copy of the orders of the engine
	 */
	List<Order> getAllOrders(){
		//defensive copy; nobody can touch the original list
		List<Order> allOrdersCopy = new ArrayList<Order>(this.allOrders);
		return allOrdersCopy;
	}
	//	public PizzariaEngine(ArrayList<Order> allOrders) {
	//		this.allOrders = allOrders;
	//	}
}//end class
